import { Expect, Test, Setup} from "alsatian";
import { Chessboard, createEmptyChessboard, putPiece } from '../../main/ts/chessboard';
import { queenMove } from '../../main/ts/move-validation';
import { Position, position } from '../../main/ts/position';
import { Move, move } from '../../main/ts/movements';

let chessboard : Chessboard;

const positionA4: Position = position(0, 3) // A4
const positionA5: Position = position(0, 4) // A5
const positionA6: Position = position(0, 5) // A6
const positionA7: Position = position(0, 6) // A7
const positionA8: Position = position(0, 7) // A8

const positionB1: Position = position(1, 0) // B1
const positionB2: Position = position(1, 1) // B2
const positionB3: Position = position(1, 2) // B3
const positionB5: Position = position(1, 4) // B5
const positionB6: Position = position(1, 5) // B6

const positionC2: Position = position(2, 1) // C2
const positionC3: Position = position(2, 2) // C3
const positionC4: Position = position(2, 3) // C4
const positionC5: Position = position(2, 4) // C5
const positionC6: Position = position(2, 5) // C6
const positionC7: Position = position(2, 6) // C7

const positionD1: Position = position(3, 0) // D1
const positionD2: Position = position(3, 1) // D2
const positionD3: Position = position(3, 2) // D3
const positionD4: Position = position(3, 3) // D4
const positionD5: Position = position(3, 4) // D5
const positionD6: Position = position(3, 5) // D6
const positionD7: Position = position(3, 6) // D7


const positionE1: Position = position(4, 0) // E1
const positionE2: Position = position(4, 1) // E2
const positionE3: Position = position(4, 2) // E3
const positionE4: Position = position(4, 3) // E4
const positionE5: Position = position(4, 4) // E5
const positionE6: Position = position(4, 5) // E6
const positionE7: Position = position(4, 6) // E7
const positionE8: Position = position(4, 7) // E8

const positionF1: Position = position(5, 0) // F1
const positionF2: Position = position(5, 1) // F2
const positionF3: Position = position(5, 2) // F3
const positionF4: Position = position(5, 3) // F4
const positionF5: Position = position(5, 4) // F5
const positionF6: Position = position(5, 5) // F6
const positionF7: Position = position(5, 6) // F7

const positionG1: Position = position(6, 0) // G1
const positionG2: Position = position(6, 1) // G2
const positionG3: Position = position(6, 2) // G3
const positionG4: Position = position(6, 3) // G4
const positionG5: Position = position(6, 4) // G5
const positionG6: Position = position(6, 5) // G6

const positionH1: Position = position(7, 0) // H1
const positionH3: Position = position(7, 2) // H3
const positionH4: Position = position(7, 3) // H4
const positionH5: Position = position(7, 4) // H5
const positionH7: Position = position(7, 6) // H7

// Horizontal moves
const moveE4_H4: Move = move(positionE4, positionH4);
const moveE4_A4: Move = move(positionE4, positionA4);

// Vertical moves
const moveE4_E1: Move = move(positionE4, positionE1);
const moveE4_E8: Move = move(positionE4, positionE8);

// Diagonal moves
const moveE4_A8: Move = move(positionE4, positionA8);
const moveE4_B1: Move = move(positionE4, positionB1);
const moveE4_H7: Move = move(positionE4, positionH7);
const moveE4_H1: Move = move(positionE4, positionH1);

// Camel moves
const moveE4_F7: Move = move(positionE4, positionF7);
const moveE4_H5: Move = move(positionE4, positionH5);
const moveE4_F1: Move = move(positionE4, positionF1);
const moveE4_H3: Move = move(positionE4, positionH3);
const moveE4_D1: Move = move(positionE4, positionD1);
const moveE4_B3: Move = move(positionE4, positionB3);
const moveE4_B5: Move = move(positionE4, positionB5);
const moveE4_D7: Move = move(positionE4, positionD7);

// Impossible moves
const moveE4_C7: Move = move(positionE4, positionC7);
const moveE4_B2: Move = move(positionE4, positionB2);

export class TestQueenMoves {
    @Setup
    beforeEach() {
        chessboard = createEmptyChessboard();
        putPiece(chessboard, positionE4, { symbol: "♕", name: "White Queen", isWhite: true });

    }

    @Test("A Queen can move diagonally")
    testCanMoveDiagonally() {
        Expect(queenMove(chessboard, moveE4_A8)).toBeTruthy();

        Expect(queenMove(chessboard, moveE4_B1)).toBeTruthy();

        Expect(queenMove(chessboard, moveE4_H7)).toBeTruthy();

        Expect(queenMove(chessboard, moveE4_H1)).toBeTruthy();       
    }

    @Test("A Queen can move horizontally")
    testCanMoveHorizontally() {
        Expect(queenMove(chessboard, moveE4_H4)).toBeTruthy();

        Expect(queenMove(chessboard, moveE4_A4)).toBeTruthy();
    }

    @Test("A Queen can move vertically")
    testCanMoveVertically() {
        Expect(queenMove(chessboard, moveE4_E1)).toBeTruthy();

        Expect(queenMove(chessboard, moveE4_E8)).toBeTruthy();
    }

    @Test("A Queen can only move horizontally, vertically, and diagonally")
    testForbiddenMoves() {
        Expect(queenMove(chessboard, moveE4_C7)).not.toBeTruthy();

        Expect(queenMove(chessboard, moveE4_B2)).not.toBeTruthy();
    }

    @Test("A Queen cannot leap other pieces")
    testCannotLeap() {
        putPiece(chessboard, positionC6, { symbol: "♙", name: "White Pawn", isWhite: true });
        putPiece(chessboard, positionF4, { symbol: "♟", name: "Black Pawn", isWhite: false });
        Expect(queenMove(chessboard, moveE4_A8)).not.toBeTruthy();

        Expect(queenMove(chessboard, moveE4_H4)).not.toBeTruthy();
    }

    @Test("A Queen cannot capure pieces from the same color")
    testCannotCaptureSameColor() {
        putPiece(chessboard, positionH4, { symbol: "♙", name: "White Pawn", isWhite: true });
        Expect(queenMove(chessboard, moveE4_H4)).not.toBeTruthy();      
    }

    @Test("A Queen can capure pieces from a different color")
    testCanCaptureDifferentColor() {
        putPiece(chessboard, positionH4, { symbol: "♟", name: "Black Pawn", isWhite: false });
        Expect(queenMove(chessboard, moveE4_H4)).toBeTruthy();       
    }
}